@@ -109,7 +109,7 @@ sub get_product_name()
 
 sub get_product_version()
 {
-    return "21.2.0.33926";
+    return "22.1.1.35679";
 }
 
 sub get_std_icon($)
@@ -362,16 +362,13 @@ sub expand_cmdline($)
 sub shquote_string($)
 {
     my ($str)=@_;
-    $str =~ s%\\%\\\\%g;
-    $str =~ s%\$%\\\$%g;
-    $str =~ s%\"%\\\"%g;
-    $str =~ s%\`%\\\`%g;
+    $str =~ s~([\\\$"`])~\\$1~g;
     return "\"$str\"";
 }
 
 sub argv2shcmd(@)
 {
-    return join(' ', map { /[^a-zA-Z0-9\/.,+_-]/ ? shquote_string($_) : $_ } @_);
+    return join(' ', map { /[^a-zA-Z0-9\/.,+_:=-]/ ? shquote_string($_) : $_ } @_);
 }
 
 
