@@ -3,11 +3,11 @@
 import os
 import fcntl
 import ntpath
+import platform
 import shutil
 import subprocess
 import threading
 import time
-import traceback
 
 import distversion
 
@@ -74,46 +74,6 @@ def get_bottle_list():
 
     return list(allbottles)
 
-def get_games_bottle_list():
-
-    allbottles = set()
-
-    for dirpath in cxproduct.get_games_bottle_path().split(":"):
-        if not os.path.exists(dirpath):
-            continue
-        for bottlecandidate in os.listdir(cxutils.string_to_unicode(dirpath)):
-            if bottlecandidate.lower() == "default":
-                continue
-            if bottlecandidate.lower().startswith(registrationBottlePrefix.lower()):
-                continue
-            if not _validate_wineprefix(os.path.join(dirpath, bottlecandidate)):
-                continue
-
-            allbottles.add(bottlecandidate)
-
-    return list(allbottles)
-
-
-
-def unique_bottle_key(inBasename):
-    # unique_bottle_name, below, provides a unique name for an active bottle in the bottledir.
-    # unique_bottle_key does that but also avoids collisions with pending games bottles.
-    existingBottles = get_bottle_list() + get_games_bottle_list()
-
-    collision = True
-    candidate = inBasename
-    i = 1
-
-    while collision:
-        collision = False
-        for bottlename in existingBottles:
-            if candidate.lower() == bottlename.lower():
-                collision = True
-                i += 1
-                candidate = inBasename + "-" + str(i)
-                break
-
-    return candidate
 
 def unique_bottle_name(inBasename):
     existingBottles = get_bottle_list()
@@ -139,7 +99,6 @@ def unique_bottle_name(inBasename):
     return candidate
 
 
-
 @cxobjc.method(BottleQuery, 'defaultBottle')
 def get_default_bottle():
 
@@ -151,7 +110,7 @@ def get_default_bottle():
             if os.path.exists(defaultcandidate):
                 return cxutils.string_to_unicode(os.path.basename(os.path.realpath(defaultcandidate)))
 
-    return u""
+    return ""
 
 
 @cxobjc.method(BottleQuery, 'prefixForBottle_')
@@ -224,7 +183,6 @@ def get_native_paths(bottlename, windowspaths):
     return nativePaths
 
 
-@cxobjc.method(BottleQuery, 'forBottle_getNativePath_')
 def get_native_path(bottlename, windowspath):
     return get_native_paths(bottlename, [windowspath])[0]
 
@@ -324,7 +282,7 @@ def get_control_panel_info(bottlename, windowsdir=None):
                                     icon_path])
 
     # Special case:  Add winecfg
-    AppletTable.append(["winecfg.exe.so", _("Wine Configuration"), _("Built-in tool for configuring Bottle Settings"), "winecfg"])
+    AppletTable.append(["winecfg.exe.so", _("Wine Configuration"), _("Built-in tool for configuring bottle settings"), "winecfg"])
 
     # Task Manager...
     AppletTable.append(["taskmgr.exe.so", _("Task Manager"), _("Manage the applications and processes running in this bottle"), "winetaskmgr"])
@@ -335,15 +293,6 @@ def get_control_panel_info(bottlename, windowsdir=None):
     return AppletTable
 
 
-@cxobjc.method(BottleQuery, 'controlPanelInfoForBottle_')
-def _controlPanelInfoForBottle_(bottlename):
-    # FIXME: only the exception handling differs -> merge with get_control_panel_info()
-    try:
-        return get_control_panel_info(bottlename)
-    except:
-        traceback.print_exc()
-        return False
-
 def config_file_path(bottlename):
     wineprefix = get_prefix_for_bottle(bottlename)
     return os.path.join(wineprefix, "cxbottle.conf")
@@ -421,17 +370,7 @@ def get_appid(bottlename):
     except NotFoundError:
         return None
 
-@cxobjc.method(BottleQuery, 'propertiesForBottle_')
-def _propertiesForBottle_(bottlename):
-    # FIXME: only the exception handling differs -> merge with get_bottle_properties()
-    try:
-        return get_bottle_properties(bottlename)
-    except:
-        traceback.print_exc()
-        return None
 
-
-@cxobjc.method(BottleQuery, 'resolveIconPath_forBottle_')
 def resolve_icon_path(iconPath, bottlename):
     manipulator = BottleManipulator(bottlename)
     windowspath = manipulator.send('getwindowsdir').get()
@@ -479,7 +418,7 @@ class AsyncManipResult(cxevent.AsyncResult):
 
 _BottleManipulator_cache = {}
 
-class _BottleManipulator(object):
+class _BottleManipulator:
     def __init__(self, bottlename):
         self.bottlename = bottlename
 
@@ -521,7 +460,7 @@ class _BottleManipulator(object):
 
             cxlog.log("starting command %s in %s" % (' '.join(cxlog.debug_str(x) for x in args), cxlog.to_str(self.bottlename)))
 
-            self.subp = subprocess.Popen(args, stdin=subprocess.PIPE,
+            self.subp = subprocess.Popen(args, stdin=subprocess.PIPE, # pylint: disable=R1732
                                          stdout=subprocess.PIPE,
                                          close_fds=True)
             # use the fd's for stdout and stdin directly to avoid caching
@@ -640,17 +579,11 @@ class _BottleManipulator(object):
             if err:
                 return
             if writable:
-                self.lock.acquire()
-                try:
+                with self.lock:
                     self._write()
-                finally:
-                    self.lock.release()
             if readable:
-                self.lock.acquire()
-                try:
+                with self.lock:
                     self._read()
-                finally:
-                    self.lock.release()
 
     def _thread_proc(self, subp):
         timeout = 15.0 # end the cxmanip process after 15 seconds of inactivity
@@ -658,13 +591,10 @@ class _BottleManipulator(object):
         while self.started:
             self.wait(r=None, timeout=timeout)
 
-            self.lock.acquire()
-            try:
+            with self.lock:
                 if not self.results and time.time() - self._last_activity_time > timeout:
                     self._close()
                     break
-            finally:
-                self.lock.release()
         subp.wait()
 
     def _send_command(self, string):
@@ -682,8 +612,7 @@ class _BottleManipulator(object):
     def send(self, string):
         cxlog.log_('manip', "%s: -> %s" % (cxlog.to_str(self.bottlename), cxlog.debug_str(string)))
         string = cxutils.string_to_utf8(string)
-        self.lock.acquire()
-        try:
+        with self.lock:
             result = self._send_command(string)
             if result is None:
                 self._start()
@@ -691,8 +620,6 @@ class _BottleManipulator(object):
                 if result is None:
                     result = AsyncManipResult(self)
                     result.set_error(IOError("Could not start the cxmanip process"))
-        finally:
-            self.lock.release()
         return result
 
     def _close(self):
@@ -704,22 +631,16 @@ class _BottleManipulator(object):
             cxlog.log("shutting down cxmanip process in %s" % cxlog.to_str(self.bottlename))
 
     def close(self):
-        self.lock.acquire()
-        try:
+        with self.lock:
             self._close()
-        finally:
-            self.lock.release()
 
     def clean_shutdown(self):
         """cleanly shut down the cxmanip process when there are no pending requests"""
-        self.lock.acquire()
-        try:
+        with self.lock:
             if not self.results:
                 self._close()
             else:
                 self.closing = True
-        finally:
-            self.lock.release()
 
     def __del__(self):
         self.close()
@@ -776,7 +697,7 @@ def deserialize_reg(lines):
             raise Exception("got unexpected data: %s" % line)
     return subkeys, values
 
-@cxobjc.method(BottleQuery, 'getRegistryKeyInBottle_atPath_')
+
 def get_registry_key(bottlename, path, arch=REG_NATIVE):
     manipulator = BottleManipulator(bottlename)
 
@@ -794,7 +715,6 @@ def get_registry_key(bottlename, path, arch=REG_NATIVE):
     return deserialize_reg(serialdata.split('\n'))
 
 
-@cxobjc.method(BottleQuery, 'setRegistryKeyInBottle_atPath_withName_andValue_')
 def set_registry_key(bottlename, path, name, value, arch=REG_NATIVE):
     manipulator = BottleManipulator(bottlename)
 
@@ -886,8 +806,8 @@ _settable_versions = {
     "winxpsp2":  ("Windows XP",        5,  1, 0xA28, 2, "Service Pack 2", 2, 0, "WinNT"),
     }
 _settable_versions.update(_windows_versions)
-for nickname in version_nicknames:
-    _settable_versions[nickname] = _windows_versions[version_nicknames[nickname]]
+for nickname, version in version_nicknames.items():
+    _settable_versions[nickname] = _windows_versions[version]
 
 def set_windows_version(bottlename, version):
     info = _settable_versions[version]
@@ -939,8 +859,7 @@ def get_windows_version(bottlename):
         _subkeys, values = get_registry_key(bottlename, 'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion')
         if 'versionnumber' in values:
             version_numbers = tuple(int(x) for x in values['versionnumber'].split('.'))
-            for version in _windows_versions:
-                version_info = _windows_versions[version]
+            for version, version_info in _windows_versions.items():
                 if version_info[4] == 1 and version_info[1:4] == version_numbers:
                     return version
     except NotFoundError:
@@ -951,8 +870,7 @@ def get_windows_version(bottlename):
         _subkeys, prod_values = get_registry_key(bottlename, 'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\ProductOptions')
         if 'currentversion' in values and 'producttype' in prod_values:
             version_numbers = tuple(int(x) for x in values['currentversion'].split('.'))
-            for version in _windows_versions:
-                version_info = _windows_versions[version]
+            for version, version_info in _windows_versions.items():
                 if version_info[4] == 2 and version_info[1:3] == version_numbers and version_info[8] == prod_values['producttype']:
                     return version
     except NotFoundError:
@@ -1098,6 +1016,17 @@ def get_dxvkfiles_filename(bottlename):
 def get_dxvkfiles_config(bottlename):
     return cxconfig.File(get_dxvkfiles_filename(bottlename))
 
+def get_32bit_dxvk_available():
+    # Don't enable DXVK for 32-bit apps on macOS 10.14 and before, because 32-bit Vulkan will never work.
+    mac_ver = platform.mac_ver()[0]
+    return not distversion.IS_MACOSX or not mac_ver.startswith('10') or mac_ver.startswith('10.15')
+
+def get_dxvk_available(bottlename):
+    if not distversion.IS_MACOSX or get_bottle_properties(bottlename)['arch'] == 'win64':
+        return True
+
+    return get_32bit_dxvk_available()
+
 def _copy_dll_list(bottlename, srcdir, dst_windir, basenames, dxvkfiles_config=None):
     for dll in basenames:
         dll = dll + '.dll'
@@ -1108,10 +1037,8 @@ def _copy_dll_list(bottlename, srcdir, dst_windir, basenames, dxvkfiles_config=N
             if dxvkfiles_config is not None:
                 checksum = cxutils.md5sum_path(src)
             shutil.copyfile(src, dst)
-        except Exception as e:
-            if cxutils.is_file_not_found(e):
-                continue
-            raise
+        except FileNotFoundError:
+            continue
         if dxvkfiles_config is not None:
             dxvkfiles_config['DxvkFiles'][dst_winpath] = checksum
 
@@ -1127,9 +1054,7 @@ def enable_dxvk(bottlename):
         lib64_path = os.path.join(cxutils.CX_ROOT, 'lib64', 'wine', 'dxvk')
         _copy_dll_list(bottlename, lib64_path, sys64_path, _dxvk_dlls, configfile)
 
-    # Don't enable DXVK for 32-bit apps on macOS.
-    # Vulkan doesn't currently work under 32on64, and will never work on pure 32-bit (10.14 and before).
-    if not distversion.IS_MACOSX:
+    if get_32bit_dxvk_available():
         lib32_path = os.path.join(cxutils.CX_ROOT, 'lib', 'wine', 'dxvk')
         _copy_dll_list(bottlename, lib32_path, sys32_path, _dxvk_dlls, configfile)
 
@@ -1148,15 +1073,14 @@ def disable_dxvk(bottlename):
     win_environ = get_win_environ(bottlename, ('winsysdir', 'winsysdir64'))
     sys64_path = expand_win_string(win_environ, "%WinSysDir64%")
     sys32_path = expand_win_string(win_environ, "%WinSysDir%")
+    lib_path = os.path.join(cxutils.CX_ROOT, 'lib', 'wine')
 
     if sys64_path != sys32_path:
-        lib64_path = os.path.join(cxutils.CX_ROOT, 'lib64', 'wine')
+        lib64_path = os.path.join(lib_path, 'x86_64-windows')
         _copy_dll_list(bottlename, lib64_path, sys64_path, _dxvk_dlls)
-        _copy_dll_list(bottlename, os.path.join(lib64_path, 'fakedlls'), sys64_path, _dxvk_dlls)
 
-    lib32_path = os.path.join(cxutils.CX_ROOT, 'lib', 'wine')
+    lib32_path = os.path.join(lib_path, 'i386-windows')
     _copy_dll_list(bottlename, lib32_path, sys32_path, _dxvk_dlls)
-    _copy_dll_list(bottlename, os.path.join(lib32_path, 'fakedlls'), sys32_path, _dxvk_dlls)
 
     for dll in _dxvk_dlls:
         unset_registry_value(bottlename,
@@ -1165,6 +1089,5 @@ def disable_dxvk(bottlename):
 
     try:
         os.remove(get_dxvkfiles_filename(bottlename))
-    except Exception as e:
-        if not cxutils.is_file_not_found(e):
-            raise
+    except FileNotFoundError:
+        pass
