@@ -4,34 +4,32 @@ import os
 
 import distversion
 
-import bottlewrapperbase
 import bottlequery
 import bottlemanagement
 import c4profiles
+import c4profilesmanager
+import cxdiag
 import cxobjc
 import cxutils
 
 # for localization
 from cxutils import cxgettext as _
 
-class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
 
-    STATUS_INIT = _(u"Scanning\u2026")
-    STATUS_UPGRADE = _(u"Needs upgrade")
-    STATUS_READY = _(u"Ready")
-    STATUS_ARCHIVING = _(u"Archiving\u2026")
-    STATUS_DEFAULTING = _(u"Making default\u2026")
-    STATUS_RENAMING = _(u"Renaming\u2026")
-    STATUS_DELETING = _(u"Deleting\u2026")
-    STATUS_DOWNING = _(u"Shutting down\u2026")
-    STATUS_FORCE_DOWNING = _(u"Forcing shutdown\u2026")
-    STATUS_DOWN = _(u"Shut down")
-    STATUS_UPGRADING = _(u"Upgrading\u2026")
-    STATUS_REPAIRING = _(u"Repairing\u2026")
+class BottleWrapper(cxobjc.Proxy):
 
-    STATUS_X11_UNKNOWN = "X11Uknown"
-    STATUS_X11_YES = "X11Yes"
-    STATUS_X11_NO = "X11No"
+    STATUS_INIT = _("Scanning…")
+    STATUS_UPGRADE = _("Needs upgrade")
+    STATUS_READY = _("Ready")
+    STATUS_ARCHIVING = _("Archiving…")
+    STATUS_DEFAULTING = _("Making default…")
+    STATUS_RENAMING = _("Renaming…")
+    STATUS_DELETING = _("Deleting…")
+    STATUS_DOWNING = _("Shutting down…")
+    STATUS_FORCE_DOWNING = _("Forcing shutdown…")
+    STATUS_DOWN = _("Shut down")
+    STATUS_UPGRADING = _("Upgrading…")
+    STATUS_REPAIRING = _("Repairing…")
 
     STATUS_CSMT_UNKNOWN = "CSMTUnknown"
     STATUS_CSMT_ENABLED = "CSMTEnabled"
@@ -41,14 +39,17 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
     STATUS_DXVK_ENABLED = "DXVKEnabled"
     STATUS_DXVK_CUSTOM = "DXVKCustom"
     STATUS_DXVK_DISABLED = "DXVKDisabled"
+    STATUS_DXVK_UNAVAILABLE = "DXVKUnavailable"
+
+    STATUS_ESYNC_UNKNOWN = "ESyncUnknown"
+    STATUS_ESYNC_ENABLED = "ESyncEnabled"
+    STATUS_ESYNC_CUSTOM = "ESyncCustom"
+    STATUS_ESYNC_DISABLED = "ESyncDisabled"
 
     STATUS_HIGHRES_UNKNOWN = "HIGHRESUnknown"
     STATUS_HIGHRES_ENABLED = "HIGHRESEnabled"
     STATUS_HIGHRES_DISABLED = "HIGHRESDisabled"
-
-    STATUS_ESYNC_UNKNOWN = "ESYNCUnknown"
-    STATUS_ESYNC_ENABLED = "ESYNCEnabled"
-    STATUS_ESYNC_DISABLED = "ESYNCDisabled"
+    STATUS_HIGHRES_UNAVAILABLE = "HIGHRESUnavailable"
 
     # versions of DXVK shipped with CrossOver before tracking hashes
     _builtin_dxvk_hashes = {
@@ -101,8 +102,12 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
     }
 
     def __init__(self, inName):
+        cxobjc.Proxy.__init__(self)
+
         cxutils.expect_unicode(inName)
-        bottlewrapperbase.BottleWrapperBase.__init__(self, inName)
+        self.name = inName
+        self.changeablename = inName
+
         self._change_delegates = []
         self._bottle_info_ready = False
         self._property_dict = {}
@@ -117,11 +122,6 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         self._control_panel_table = []
         self._control_panel_off_thread_table = []
 
-
-        self.is_x11_driver_ready = False
-        self.is_x11_driver_state = BottleWrapper.STATUS_X11_UNKNOWN
-        self._graphicsDriverKey = "HKEY_CURRENT_USER\\Software\\Wine\\Drivers"
-
         self.is_csmt_disabled_ready = False
         self.is_csmt_disabled_loading = False
         self.is_csmt_disabled_state = BottleWrapper.STATUS_CSMT_UNKNOWN
@@ -132,14 +132,14 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         self._dlloverridesKey = "HKEY_CURRENT_USER\\Software\\Wine\\DllOverrides"
         self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_UNKNOWN
 
-        self.is_high_resolution_enabled_ready = False
-        self.is_high_resolution_enabled_loading = False
-        self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_UNKNOWN
-
         self.is_esync_enabled_ready = False
         self.is_esync_enabled_loading = False
         self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_UNKNOWN
 
+        self.is_high_resolution_enabled_ready = False
+        self.is_high_resolution_enabled_loading = False
+        self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_UNKNOWN
+
         self.is_high_resolution_ready = False
         self.is_high_resolution_state = False
         self._macDriverKey = "HKEY_CURRENT_USER\\Software\\Wine\\Mac Driver"
@@ -168,7 +168,6 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         self.is_managed = bottlequery.is_managed(self.name)
         self.refresh_up_to_date()
 
-
         self.marked_for_death = False
 
         # Let's learn a bit about ourselves. Everything here
@@ -181,6 +180,13 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         self.load_basic_info()
         self.post_load_basic_info()
 
+    # This is a special initializer for objc. It must always be called
+    #  explicitly on the mac.
+    def initWithName_(self, inName):
+        self = cxobjc.Proxy.nsobject_init(self)
+        if self is not None:
+            self.__init__(inName)
+        return self
 
     # This will be called, below, only on ObjC. It sets up
     #  some miscellaneous notifications that the GUI needs.
@@ -200,7 +206,22 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         cls.setKeys_triggerChangeNotificationsForDependentKey_(["status", "marked_for_death", "is_managed"], "can_rename")
         cls.setKeys_triggerChangeNotificationsForDependentKey_(["up_to_date", "is_managed"], "needs_upgrade")
 
+    def bottle_changed(self):
+        for delegate in self._change_delegates:
+            delegate.bottleChanged(self)
 
+    # Setters that call delegates as well as setting the variable. Not turning those into properties
+    # allows us to not manually handle KVO for macOS
+
+    @cxobjc.python_method
+    def set_is_managed(self, is_managed):
+        self.is_managed = is_managed
+        self.bottle_changed()
+
+    @cxobjc.python_method
+    def set_marked_for_death(self, marked_for_death):
+        self.marked_for_death = marked_for_death
+        self.bottle_changed()
 
     #####
     #
@@ -241,9 +262,9 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
             self.bottletemplate = self._property_dict["template"]
 
             if "managed" in self._property_dict:
-                self.is_managed = self._property_dict["managed"]
+                self.set_is_managed(self._property_dict["managed"])
             else:
-                self.is_managed = False
+                self.set_is_managed(False)
 
             self.arch = self._property_dict["arch"]
 
@@ -274,59 +295,6 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
             self.mtime = self._real_mtime()
             self._needs_refresh_up_to_date = True
 
-    #####
-    #
-    # load x11 driver preferences
-    #
-    # Three sections, same as load_basic_info, above.
-    #
-    # Loads the setting which determines whether
-    # the x11 driver is enabled or not.
-    #
-    #####
-
-    @cxobjc.namedSelector(b'preLoadIsX11Driver')
-    def pre_load_is_x11_driver(self):
-        self.is_x11_driver_ready = False
-
-    @cxobjc.namedSelector(b'loadIsX11Driver')
-    def load_is_x11_driver(self):
-
-        self.is_x11_driver_state = BottleWrapper.STATUS_X11_NO
-
-
-
-        try:
-            _subkeys, values = bottlequery.get_registry_key(self.name,
-                                                            self._graphicsDriverKey)
-            if 'graphics' in values:
-                if values['graphics'].upper().find("X11") == 0:
-                    self.is_x11_driver_state = BottleWrapper.STATUS_X11_YES
-        except bottlequery.NotFoundError:
-            pass
-
-    @cxobjc.namedSelector(b'postLoadIsX11Driver')
-    def post_load_is_x11_driver(self):
-        self.is_x11_driver_ready = True
-
-    @cxobjc.namedSelector(b'enableX11Driver')
-    def enable_x11_driver(self):
-        bottlequery.set_registry_key(self.name,
-                                     self._graphicsDriverKey,
-                                     "Graphics",
-                                     "x11,mac")
-        self.is_x11_driver_state = BottleWrapper.STATUS_X11_YES
-
-    @cxobjc.namedSelector(b'disableX11Driver')
-    def disable_x11_driver(self):
-        bottlequery.unset_registry_value(self.name,
-                                         self._graphicsDriverKey,
-                                         "Graphics")
-        self.is_x11_driver_state = BottleWrapper.STATUS_X11_NO
-
-    def is_x11_driver(self):
-        return self.is_x11_driver_state == BottleWrapper.STATUS_X11_YES
-
     #####
     #
     # load dxvk preferences
@@ -347,63 +315,76 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         self.is_dxvk_enabled_ready = False
         self.is_dxvk_enabled_loading = True
 
+        self.bottle_changed()
+
     @cxobjc.namedSelector(b'loadIsDXVKEnabled')
     def load_is_dxvk_enabled(self):
+        if not bottlequery.get_dxvk_available(self.name):
+            self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_UNAVAILABLE
+            return
 
         self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_UNKNOWN
 
         try:
-
-            self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_DISABLED
-
-            _subkeys, values = bottlequery.get_registry_key(self.name,
-                                                            self._dlloverridesKey)
+            _subkeys, values = bottlequery.get_registry_key(self.name, self._dlloverridesKey)
 
             if 'd3d11' in values:
                 if values['d3d11'].startswith('native'):
                     self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_CUSTOM
-                    d3d11_winpath = bottlequery.expand_win_string(bottlequery.get_win_environ(self.name, c4profiles.ENVIRONMENT_VARIABLES), "%WinSysDir64%\\d3d11.dll")
+                    d3d11_winpath = bottlequery.expand_win_string(bottlequery.get_win_environ(
+                        self.name, c4profiles.ENVIRONMENT_VARIABLES), "%WinSysDir64%\\d3d11.dll")
                     d3d11_path = bottlequery.get_native_path(self.name, d3d11_winpath)
                     checksum = cxutils.md5sum_path(d3d11_path)
 
                     if checksum in self._builtin_dxvk_hashes or \
-                        checksum == self.get_dxvkfiles_config()['DxvkFiles'].get(d3d11_winpath, ''):
+                       checksum == self.get_dxvkfiles_config()['DxvkFiles'].get(d3d11_winpath, ''):
                         self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_ENABLED
         except bottlequery.NotFoundError:
             pass
 
+        if self.is_dxvk_enabled_state == BottleWrapper.STATUS_DXVK_UNKNOWN:
+            self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_DISABLED
+
     @cxobjc.namedSelector(b'postLoadIsDXVKEnabled')
     def post_load_is_dxvk_enabled(self):
         self.is_dxvk_enabled_ready = True
         self.is_dxvk_enabled_loading = False
 
+        self.bottle_changed()
+
     @cxobjc.namedSelector(b'enableDXVK')
     def enable_dxvk(self):
         bottlequery.enable_dxvk(self.name)
         self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_ENABLED
 
+        self.bottle_changed()
+
     @cxobjc.namedSelector(b'disableDXVK')
     def disable_dxvk(self):
         bottlequery.disable_dxvk(self.name)
         self.is_dxvk_enabled_state = BottleWrapper.STATUS_DXVK_DISABLED
 
+        self.bottle_changed()
+
     def is_dxvk_enabled(self):
         return self.is_dxvk_enabled_state == BottleWrapper.STATUS_DXVK_ENABLED
 
+    def is_dxvk_available(self):
+        return self.is_dxvk_enabled_state != BottleWrapper.STATUS_DXVK_UNAVAILABLE
 
     @cxobjc.namedSelector(b'preLoadIsCSMTDisabled')
     def pre_load_is_csmt_disabled(self):
         self.is_csmt_disabled_ready = False
         self.is_csmt_disabled_loading = True
 
+        self.bottle_changed()
+
     @cxobjc.namedSelector(b'loadIsCSMTDisabled')
     def load_is_csmt_disabled(self):
-
-        self.is_csmt_disabled_state = BottleWrapper.STATUS_CSMT_ENABLED
+        self.is_csmt_disabled_state = BottleWrapper.STATUS_CSMT_UNKNOWN
 
         try:
-            _subkeys, values = bottlequery.get_registry_key(self.name,
-                                                            self._direct3DKey)
+            _subkeys, values = bottlequery.get_registry_key(self.name, self._direct3DKey)
 
             if 'csmt' in values:
                 if isinstance(values['csmt'], int):
@@ -416,11 +397,16 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         except bottlequery.NotFoundError:
             pass
 
+        if self.is_csmt_disabled_state == BottleWrapper.STATUS_CSMT_UNKNOWN:
+            self.is_csmt_disabled_state = BottleWrapper.STATUS_CSMT_ENABLED
+
     @cxobjc.namedSelector(b'postLoadIsCSMTDisabled')
     def post_load_is_csmt_disabled(self):
         self.is_csmt_disabled_ready = True
         self.is_csmt_disabled_loading = False
 
+        self.bottle_changed()
+
     @cxobjc.namedSelector(b'enableCSMT')
     def enable_csmt(self):
         bottlequery.set_registry_key(self.name,
@@ -429,6 +415,8 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
                                      'dword:1')
         self.is_csmt_disabled_state = BottleWrapper.STATUS_CSMT_ENABLED
 
+        self.bottle_changed()
+
     @cxobjc.namedSelector(b'disableCSMT')
     def disable_csmt(self):
         bottlequery.set_registry_key(self.name,
@@ -437,41 +425,94 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
                                      'dword:0')
         self.is_csmt_disabled_state = BottleWrapper.STATUS_CSMT_DISABLED
 
+        self.bottle_changed()
+
     def is_csmt_disabled(self):
         return self.is_csmt_disabled_state == BottleWrapper.STATUS_CSMT_DISABLED
 
 
+    @cxobjc.namedSelector(b'preLoadIsESyncEnabled')
+    def pre_load_is_esync_enabled(self):
+        self.is_esync_enabled_ready = False
+        self.is_esync_enabled_loading = True
+
+        self.bottle_changed()
+
+    @cxobjc.namedSelector(b'loadIsESyncEnabled')
+    def load_is_esync_enabled(self):
+        self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_UNKNOWN
+
+        enabled = bottlequery.get_config_value(
+            self.name, "EnvironmentVariables", "WINEESYNC", "0") != "0"
+        if enabled:
+            self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_ENABLED
+        else:
+            self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_DISABLED
+
+    @cxobjc.namedSelector(b'postLoadIsESyncEnabled')
+    def post_load_is_esync_enabled(self):
+        self.is_esync_enabled_ready = True
+        self.is_esync_enabled_loading = False
+
+        self.bottle_changed()
+
+    @cxobjc.namedSelector(b'enableESync')
+    def enable_esync(self):
+        bottlequery.set_config_value(
+            self.name, "EnvironmentVariables", "WINEESYNC", "1")
+        self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_ENABLED
+
+        self.bottle_changed()
+
+    @cxobjc.namedSelector(b'disableESync')
+    def disable_esync(self):
+        bottlequery.set_config_value(
+            self.name, "EnvironmentVariables", "WINEESYNC", "0")
+        self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_DISABLED
+
+        self.bottle_changed()
+
+    def is_esync_enabled(self):
+        return self.is_esync_enabled_state == BottleWrapper.STATUS_ESYNC_ENABLED
+
+
     @cxobjc.python_method
     def pre_load_is_high_resolution_enabled(self):
         self.is_high_resolution_enabled_ready = False
         self.is_high_resolution_enabled_loading = True
 
+        self.bottle_changed()
+
     @cxobjc.python_method
     def load_is_high_resolution_enabled(self):
+        self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_UNKNOWN
 
-        self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_DISABLED
+        dpi = int(float(cxdiag.get(None).properties.get('display.dpi', 0)))
+        if dpi < 110:
+            self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_UNAVAILABLE
+            return
 
-        values = {}
         try:
-            _subkeys, values = bottlequery.get_registry_key(self.name,
-                                                            self._logPixelsKey)
+            _subkeys, values = bottlequery.get_registry_key(self.name, self._logPixelsKey)
+
+            if 'logpixels' not in values:
+                _subkeys, values = bottlequery.get_registry_key(self.name, self._oldLogPixelsKey)
+
+            if values.get('logpixels', 96) >= 110:
+                self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_ENABLED
         except bottlequery.NotFoundError:
             pass
-        if 'logpixels' not in values:
-            try:
-                _subkeys, values = bottlequery.get_registry_key(self.name,
-                                                                self._oldLogPixelsKey)
-            except bottlequery.NotFoundError:
-                pass
 
-        if values.get('logpixels', 96) >= 110:
-            self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_ENABLED
+        if self.is_high_resolution_enabled_state == BottleWrapper.STATUS_HIGHRES_UNKNOWN:
+            self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_DISABLED
 
     @cxobjc.python_method
     def post_load_is_high_resolution_enabled(self):
         self.is_high_resolution_enabled_ready = True
         self.is_high_resolution_enabled_loading = False
 
+        self.bottle_changed()
+
     @cxobjc.python_method
     def enable_high_resolution(self, logpixels):
         bottlequery.set_registry_key(self.name,
@@ -480,6 +521,8 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
                                      "dword:" + hex(logpixels)[2:])
         self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_ENABLED
 
+        self.bottle_changed()
+
     @cxobjc.python_method
     def disable_high_resolution(self):
         # Force the DPI to 96 here so winewrapper won't overwrite it.
@@ -489,46 +532,12 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
                                      "dword:60")
         self.is_high_resolution_enabled_state = BottleWrapper.STATUS_HIGHRES_DISABLED
 
+        self.bottle_changed()
+
     @cxobjc.python_method
     def is_high_resolution_enabled(self):
         return self.is_high_resolution_enabled_state == BottleWrapper.STATUS_HIGHRES_ENABLED
 
-
-    @cxobjc.namedSelector(b'preLoadIsESyncEnabled')
-    def pre_load_is_esync_enabled(self):
-        self.is_esync_enabled_ready = False
-        self.is_esync_enabled_loading = True
-
-    @cxobjc.namedSelector(b'loadIsESyncEnabled')
-    def load_is_esync_enabled(self):
-        self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_DISABLED
-
-        enabled = bottlequery.get_config_value(
-            self.name, "EnvironmentVariables", "WINEESYNC", "0") != "0"
-        if enabled:
-            self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_ENABLED
-
-    @cxobjc.namedSelector(b'postLoadIsESyncEnabled')
-    def post_load_is_esync_enabled(self):
-        self.is_esync_enabled_ready = True
-        self.is_esync_enabled_loading = False
-
-    @cxobjc.namedSelector(b'enableESync')
-    def enable_esync(self):
-        bottlequery.set_config_value(
-            self.name, "EnvironmentVariables", "WINEESYNC", "1")
-        self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_ENABLED
-
-    @cxobjc.namedSelector(b'disableESync')
-    def disable_esync(self):
-        bottlequery.set_config_value(
-            self.name, "EnvironmentVariables", "WINEESYNC", "0")
-        self.is_esync_enabled_state = BottleWrapper.STATUS_ESYNC_DISABLED
-
-    def is_esync_enabled(self):
-        return self.is_esync_enabled_state == BottleWrapper.STATUS_ESYNC_ENABLED
-
-
     #####
     #
     # load Mac driver high resolution mode preferences
@@ -610,7 +619,7 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
     # Three sections, same as load_basic_info, above.
     #
     # Loads the list of control panels available for this bottle,
-    # and sets up icons and other user-readable-data for each.
+    # and sets up other user-readable-data for each.
     #
     #####
 
@@ -653,32 +662,14 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         if self._needs_refresh_up_to_date:
             self.refresh_up_to_date()
 
-        icon_sizes = ['48x48', '24x24', '64x64', '32x32', '']
-        if distversion.IS_MACOSX:
-            icon_sizes.insert(0, '96x96')
-
         cptable = []
-        std_root = os.path.join(cxutils.CX_ROOT, 'share', 'icons')
         for panel in self._control_panel_off_thread_table:
             cpdict = {}
             cpdict["exe"] = panel[0]
             cpdict["name"] = panel[1]
             cpdict["description"] = panel[2]
-            base, ext = os.path.splitext(panel[3])
-            if ext == '' and base != '':
-                # This is a builtin icon
-                cpdict["icon"] = cxutils.get_icon_path(std_root, '', base, icon_sizes)
-            elif distversion.IS_MACOSX:
-                # conveniently, cxcplinfo provides us with .ico files in
-                # addition to the .xpms which the Mac can't handle.
-                if ext.lower() == ".xpm":
-                    ext = ".ico"
-                cpdict["icon"] = base + ext
-            else:
-                cpdict["icon"] = panel[3]
-            if cpdict["icon"] is None or cpdict["icon"] == '':
-                cpdict["icon"] = cxutils.get_icon_path(std_root, '', 'crossover', icon_sizes)
             cptable.append(cpdict)
+
         self.control_panel_table = cptable
         self.control_panel_ready = True
         self.control_panel_loading = False
@@ -732,7 +723,6 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         be called on the main thread, in order to properly arrange the state of
         our properties.
         """
-        import c4profilesmanager
         import appdetector
 
         self._maybe_upgrade()
@@ -780,7 +770,7 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
             return
 
         self.add_status_override(BottleWrapper.STATUS_RENAMING)
-        self.marked_for_death = True
+        self.set_marked_for_death(True)
 
     def rename(self):
         if self.changeablename == self.name:
@@ -797,7 +787,7 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
             return
 
         self.remove_status_override(BottleWrapper.STATUS_RENAMING)
-        self.marked_for_death = False
+        self.set_marked_for_death(False)
 
         if not success:
             self.changeablename = self.name
@@ -807,6 +797,37 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         self.wine_prefix = bottlequery.get_prefix_for_bottle(self.name)
         self.system_drive = os.path.join(self.wine_prefix, 'drive_c')
 
+    @cxobjc.python_method
+    def add_change_delegate(self, delegate):
+        self._change_delegates.append(delegate)
+
+    @cxobjc.python_method
+    def remove_change_delegate(self, delegate):
+        if delegate in self._change_delegates:
+            self._change_delegates.remove(delegate)
+
+    @cxobjc.python_method
+    def get_bottle_info_ready(self):
+        return self._bottle_info_ready
+
+    @cxobjc.python_method
+    def set_bottle_info_ready(self, inBool):
+        self._bottle_info_ready = inBool
+        self.bottle_changed()
+
+    bottle_info_ready = property(get_bottle_info_ready, set_bottle_info_ready)
+
+    @cxobjc.python_method
+    def get_control_panel_ready(self):
+        return self._control_panel_ready
+
+    @cxobjc.python_method
+    def set_control_panel_ready(self, inBool):
+        self._control_panel_ready = inBool
+        self.bottle_changed()
+
+    control_panel_ready = property(get_control_panel_ready, set_control_panel_ready)
+
     @cxobjc.python_method
     def get_control_panel_table(self):
         return self._control_panel_table
@@ -814,11 +835,21 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
     @cxobjc.python_method
     def set_control_panel_table(self, in_table):
         self._control_panel_table = in_table
-        for delegate in self._change_delegates:
-            delegate.bottleChanged(self)
+        self.bottle_changed()
 
     control_panel_table = property(get_control_panel_table, set_control_panel_table)
 
+    @cxobjc.python_method
+    def get_installed_packages_ready(self):
+        return self._installed_packages_ready
+
+    @cxobjc.python_method
+    def set_installed_packages_ready(self, inBool):
+        self._installed_packages_ready = inBool
+        self.bottle_changed()
+
+    installed_packages_ready = property(get_installed_packages_ready, set_installed_packages_ready)
+
     @cxobjc.python_method
     def get_installed_packages(self):
         return self._installed_packages
@@ -829,9 +860,8 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         # installedPackageValues stores the same data but as an array.
         # We need this for bindings, and having an actual member
         # allows us to use KVO properly.
-        self.installedPackageValues = cxutils.values(installed_apps)
-        for delegate in self._change_delegates:
-            delegate.bottleChanged(self)
+        self.installedPackageValues = list(installed_apps.values())
+        self.bottle_changed()
 
     installed_packages = property(get_installed_packages, set_installed_packages)
 
@@ -847,8 +877,7 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         else:
             self.current_description = None
 
-        for delegate in self._change_delegates:
-            delegate.bottleChanged(self)
+        self.bottle_changed()
 
     def _real_mtime(self):
         config_file_path = bottlequery.config_file_path(self.name)
@@ -876,17 +905,11 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
 
     needs_update = property(get_needs_update)
 
-    def get_needs_import(self):
-        return False
-
-    needs_import = property(get_needs_import)
-
     @cxobjc.namedSelector(b'addStatusOverride:')
     def add_status_override(self, status):
         self.status_overrides.append(status)
         self.status = status
-        for delegate in self._change_delegates:
-            delegate.bottleChanged(self)
+        self.bottle_changed()
 
     @cxobjc.python_method
     def remove_all_status_overrides(self, status):
@@ -898,14 +921,12 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
         if status in self.status_overrides:
             self.status_overrides.remove(status)
         if self.status_overrides:
-            self.status = self.status_overrides[len(self.status_overrides)-1]
-        elif not self.bottle_info_ready or not self.installed_packages_ready or \
-                not (distversion.IS_MACOSX or self.control_panel_ready):
+            self.status = self.status_overrides[len(self.status_overrides) - 1]
+        elif not self.bottle_info_ready or not self.installed_packages_ready or not self.control_panel_ready:
             self.status = BottleWrapper.STATUS_INIT
         else:
             self.status = BottleWrapper.STATUS_READY
-        for delegate in self._change_delegates:
-            delegate.bottleChanged(self)
+        self.bottle_changed()
 
     def get_can_run_commands(self):
         return (self.up_to_date or not self.is_managed) and not self.is_busy
@@ -917,6 +938,17 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
 
     can_edit = property(get_can_edit)
 
+    def get_is_usable(self):
+        for status in self.status_overrides:
+            if status not in (self.STATUS_INIT, self.STATUS_UPGRADE,
+                              self.STATUS_READY, self.STATUS_DEFAULTING,
+                              self.STATUS_DOWN):
+                return False
+
+        return True
+
+    is_usable = property(get_is_usable)
+
     def canInstall(self):
         return not self.is_managed and not self.marked_for_death and \
             BottleWrapper.STATUS_RENAMING not in self.status_overrides and \
@@ -925,6 +957,8 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
             BottleWrapper.STATUS_FORCE_DOWNING not in self.status_overrides and \
             BottleWrapper.STATUS_DOWN not in self.status_overrides
 
+    can_install = property(canInstall)
+
     def get_is_active(self):
         return self.bottle_info_ready and not self.status_overrides and self.up_to_date
 
@@ -962,7 +996,7 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
     def setMarkedForDeath_(self, inBool):
         if inBool:
             self.add_status_override(BottleWrapper.STATUS_DELETING)
-        self.marked_for_death = inBool
+        self.set_marked_for_death(inBool)
 
     def is_running(self):
         return bottlemanagement.is_running(self.name)
@@ -1011,6 +1045,8 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
             else:
                 self.remove_all_status_overrides(BottleWrapper.STATUS_UPGRADE)
 
+        self.bottle_changed()
+
     def preForceQuit(self):
         """On the Mac, this function MUST be called in the main thread because
         it modifies properties which are bound to the UI.
@@ -1060,10 +1096,29 @@ class BottleWrapper(bottlewrapperbase.BottleWrapperBase):
 
     appid = property(get_appid)
 
+    def get_profile(self):
+        appid = self.appid
+        if not appid:
+            return None
+
+        appid = 'com.codeweavers.c4.%s' % appid
+        profiles = c4profilesmanager.C4ProfilesSet.all_profiles()
+        return profiles.get(appid, None)
+
+    profile = property(get_profile)
+
     # A couple of crutches for reverse-compatibility with the old BottleWrapper.m
     def bottleName(self):
         return self.name
 
+    def reveal_bottle(self):
+        '''Call this to reveal the bottle in the main UI'''
+        def do_nothing(*_args, **_kwargs):
+            pass
+
+        for delegate in self._change_delegates:
+            getattr(delegate, 'revealBottle', do_nothing)(self)
+
 
 if distversion.IS_MACOSX:
     BottleWrapper.prepare_kvo_notification()
