@@ -5,7 +5,6 @@ import os
 import os.path
 import shutil
 import stat
-import sys
 import zipfile
 import tarfile
 import tempfile
@@ -93,6 +92,7 @@ class AIECore(cxaiebase.AIETask):
         self.scan_media = None
         self.release_media = None
         self.canceled = False
+        self.interactive = True
 
         # Nearest AIECore objects with an associated media
         self.parent_media = set()
@@ -334,6 +334,11 @@ class AIECore(cxaiebase.AIETask):
         return True
 
 
+    def reset_cwd(self):
+        if 'previous_cwd' in self.state:
+            os.chdir(self.state['previous_cwd'])
+            del self.state['previous_cwd']
+
     def _main_uncompress(self):
         """Check if the installer file is an archive.
 
@@ -398,52 +403,36 @@ class AIECore(cxaiebase.AIETask):
         # save the cwd so that we can restore it later on, then
         #  point at the tempdir in case any of our archive models
         #  extract directly to the cwd.
-        previous_cwd = os.getcwd()
+        self.state['previous_cwd'] = os.getcwd()
         os.chdir(tmpdir)
 
-
         # Zip file support
         if archive_type == 'zip':
-            zipf = zipfile.ZipFile(installer_file, 'r', allowZip64=True)
-            for source in zipf.namelist():
-                if source.startswith('/') or '..' in source.split('/'):
-                    # These would be created outside the given path
-                    cxlog.warn("Skipping file %s in %s (bad filename)" % (cxlog.debug_str(source), cxlog.debug_str(installer_file)))
-                    continue
-                if self.installer.zip_encoding:
-                    encoding = self.installer.zip_encoding
-                else:
-                    encoding = 'ascii'
-                if sys.version_info >= (3,):
+            with zipfile.ZipFile(installer_file, 'r', allowZip64=True) as zipf:
+                for source in zipf.namelist():
+                    if source.startswith('/') or '..' in source.split('/'):
+                        # These would be created outside the given path
+                        cxlog.warn("Skipping file %s in %s (bad filename)" % (cxlog.debug_str(source), cxlog.debug_str(installer_file)))
+                        continue
+                    if self.installer.zip_encoding:
+                        encoding = self.installer.zip_encoding
+                    else:
+                        encoding = 'ascii'
                     # target is decoded as CP437 by the zipfile library
                     target = source.encode('CP437')
-                else:
-                    target = source
-                target = os.path.join(tmpdir, target.decode(encoding))
-                try:
-                    os.makedirs(target.rsplit('/', 1)[0])
-                except OSError:
-                    # File already exists
-                    pass
-                if not target.endswith('\\') and not target.endswith('/'):
-                    if self.installer.zip_encoding:
-                        data = zipf.read(source)
-                        dfile = open(target, 'wb')
-                        dfile.write(data)
-                        dfile.close()
-                    else:
-                        try:
-                            # With extract(), introduced in Python 2.6, we don't
-                            # have to read the whole archive member into memory
-                            # which helps a lot for large files.
-                            # pylint: disable=E1101
-                            zipf.extract(source, tmpdir)
-                        except AttributeError:
+                    target = os.path.join(tmpdir, target.decode(encoding))
+                    try:
+                        os.makedirs(target.rsplit('/', 1)[0])
+                    except OSError:
+                        # File already exists
+                        pass
+                    if not target.endswith('\\') and not target.endswith('/'):
+                        if self.installer.zip_encoding:
                             data = zipf.read(source)
-                            dfile = open(target, 'wb')
-                            dfile.write(data)
-                            dfile.close()
-            zipf.close()
+                            with open(target, 'wb') as dfile:
+                                dfile.write(data)
+                        else:
+                            zipf.extract(source, tmpdir)
 
         # Microsoft Cabinet file support
         if archive_type == 'cab':
@@ -456,43 +445,42 @@ class AIECore(cxaiebase.AIETask):
                                              stderr=cxutils.GRAB)
             if retcode:
                 self.error = err
-                os.chdir(previous_cwd)
+                self.reset_cwd()
                 return False
 
         # Tar file support
         if archive_type == 'tar':
             cxlog.log_('tar', "Extracting archive %s to %s" % (cxlog.debug_str(installer_file), cxlog.debug_str(tmpdir)))
-            tarf = tarfile.open(installer_file, 'r')
-            members = tarf.getmembers()
-            # Extract deeper files first to avoid the issues described at
-            # http://www.python.org/doc/2.5.2/lib/tarfile-objects.html#l2h-2413
-            members = sorted(members, key=tar_member_depth, reverse=True)
-            for member in members:
-                if member.name.startswith('/') or '..' in member.name.split('/'):
-                    # These would be created outside the given path
-                    cxlog.warn("Skipping file %s in %s (bad filename)" % (cxlog.debug_str(member.name), cxlog.debug_str(installer_file)))
-                    continue
-                # Make sure everything leading up to and including this file is a directory
-                path_elements = os.path.normpath(member.name).split('/')
-                bad_path = False
-                for i in range(len(path_elements)):
-                    path = os.path.join(tmpdir, '/'.join(path_elements[0:i+1]))
-                    try:
-                        mode = os.lstat(path).st_mode # use lstat because symlinks to directories are bad
-                    except OSError:
-                        # Path does not exist; this is fine
-                        break
-                    if not stat.S_ISDIR(mode):
-                        bad_path = True
-                        break
-                if bad_path:
-                    # This could be a symlink to something outside the extraction directory.
-                    # Even if it's a regular file, we don't expect to overwrite any of those.
-                    cxlog.warn("Skipping file %s in %s (not a directory)" % (cxlog.debug_str(member.name), cxlog.debug_str(installer_file)))
-                    continue
-                cxlog.log_('tar', "Extracting file %s to %s" % (cxlog.debug_str(member.name), cxlog.debug_str(tmpdir)))
-                tarf.extract(member, tmpdir)
-            tarf.close()
+            with tarfile.open(installer_file, 'r') as tarf:
+                members = tarf.getmembers()
+                # Extract deeper files first to avoid the issues described at
+                # http://www.python.org/doc/2.5.2/lib/tarfile-objects.html#l2h-2413
+                members = sorted(members, key=tar_member_depth, reverse=True)
+                for member in members:
+                    if member.name.startswith('/') or '..' in member.name.split('/'):
+                        # These would be created outside the given path
+                        cxlog.warn("Skipping file %s in %s (bad filename)" % (cxlog.debug_str(member.name), cxlog.debug_str(installer_file)))
+                        continue
+                    # Make sure everything leading up to and including this file is a directory
+                    path_elements = os.path.normpath(member.name).split('/')
+                    bad_path = False
+                    for i in range(len(path_elements)):
+                        path = os.path.join(tmpdir, '/'.join(path_elements[0:i+1]))
+                        try:
+                            mode = os.lstat(path).st_mode # use lstat because symlinks to directories are bad
+                        except OSError:
+                            # Path does not exist; this is fine
+                            break
+                        if not stat.S_ISDIR(mode):
+                            bad_path = True
+                            break
+                    if bad_path:
+                        # This could be a symlink to something outside the extraction directory.
+                        # Even if it's a regular file, we don't expect to overwrite any of those.
+                        cxlog.warn("Skipping file %s in %s (not a directory)" % (cxlog.debug_str(member.name), cxlog.debug_str(installer_file)))
+                        continue
+                    cxlog.log_('tar', "Extracting file %s to %s" % (cxlog.debug_str(member.name), cxlog.debug_str(tmpdir)))
+                    tarf.extract(member, tmpdir)
 
         # RAR file support
         if archive_type == 'rar':
@@ -505,7 +493,7 @@ class AIECore(cxaiebase.AIETask):
                                              stderr=cxutils.GRAB)
             if retcode:
                 self.error = err
-                os.chdir(previous_cwd)
+                self.reset_cwd()
                 return False
 
         # 7z and NSIS (NullSoft Scriptable Install System) file support
@@ -518,7 +506,7 @@ class AIECore(cxaiebase.AIETask):
                                              stderr=cxutils.GRAB)
             if retcode:
                 self.error = err
-                os.chdir(previous_cwd)
+                self.reset_cwd()
                 return False
 
         # Self-extracting executable
@@ -558,10 +546,11 @@ class AIECore(cxaiebase.AIETask):
                                              stderr=cxutils.GRAB)
             if retcode:
                 self.error = err
-                os.chdir(previous_cwd)
+                self.reset_cwd()
                 return False
 
-        os.chdir(previous_cwd)
+        self.reset_cwd()
+
         self.state['install_source'] = tmpdir
         del self.state['installer_file']
         return True
@@ -591,6 +580,14 @@ class AIECore(cxaiebase.AIETask):
     def _main_scan_installer(self):
         if 'installer_file' not in self.state:
             return True
+
+        if self.state['installer_file'].lower().startswith('steam:'):
+            return True
+
+        config = bottlequery.get_config(self.scheduler.installtask.bottlename)
+        if config['CrossOver'].get('AntiVirusScan', 'start') == 'never':
+            return True
+
         cmd = [os.path.join(cxutils.CX_ROOT, "bin", "cxavscan"),
                "--gui", self.state['installer_file']]
         cxlog.log_("aie", "antivirus_command=%s" % cmd)
@@ -708,14 +705,16 @@ class AIECore(cxaiebase.AIETask):
         # We cannot trust the installer's return code so we ignore it entirely.
         # FIXME: We should try to be interruptible
 
-        is_steam_install = self.state.get('installer_file', '').lower().startswith('steam:')
+        source = self.state.get('install_source')
+        is_steam_install = source and source.lower().startswith('steam:')
+
+        workdir = None
+        if source and not is_steam_install:
+            if os.path.isdir(source):
+                workdir = source
+            else:
+                workdir = os.path.dirname(source)
 
-        engine = self.scheduler
-        source = engine.installtask.GetInstallerSource()
-        if source and os.path.isdir(source):
-            workdir = source
-        else:
-            workdir = bottlequery.get_native_path(engine.installtask.bottlename, engine.expand_win_string("C:/windows/temp"))
         cxutils.system(self.state['installer_command'],
                        env=self.state['environ'], cwd=workdir,
                        background=is_steam_install)
@@ -809,7 +808,7 @@ class AIECore(cxaiebase.AIETask):
                     self.error = _("Unable to create the destination directory '%(dir)s' for '%(profile)s':\n%(error)s") % {
                         'dir': dst_dir,
                         'profile': self.name,
-                        'error': cxutils.unicode_type(ose)}
+                        'error': str(ose)}
                     return False
             if not os.path.isdir(dst_dir):
                 self.error = _("Cannot copy the %(profile)s file(s) into '%(dir)s' because it is not a directory.") % {
@@ -833,7 +832,7 @@ class AIECore(cxaiebase.AIETask):
                     self.error = _("An error occurred while copying '%(source)s' to '%(destination)s':\n%(error)s") % {
                         'source': source,
                         'destination': dst,
-                        'error': cxutils.unicode_type(ose)}
+                        'error': str(ose)}
                     return False
         return True
 
@@ -916,9 +915,12 @@ class AIECore(cxaiebase.AIETask):
 
 
     def _cleanup_delete_uncompressed_files(self):
+        self.reset_cwd()
+
         if 'temporary_source' in self.state:
             shutil.rmtree(self.state['temporary_source'])
             del self.state['temporary_source']
+
         return True
 
     def _main_check_silent_steam_install(self):
@@ -1070,8 +1072,7 @@ class AIECore(cxaiebase.AIETask):
         # Hack for bug 15593 (activation failure)
         # Don't want to use postdependency on taskkill bc
         # we want to terminate with prejudice and no user interaction
-        if self.installer.parent.appid == 'com.codeweavers.c4.14101' or \
-           self.installer.parent.appid == 'com.codeweavers.c4.7437':
+        if self.installer.parent.appid in ('com.codeweavers.c4.14101', 'com.codeweavers.c4.7437'):
             engine = self.scheduler
 
             cmd = [os.path.join(cxutils.CX_ROOT, "bin", "wineserver"),
@@ -1105,8 +1106,7 @@ class AIECore(cxaiebase.AIETask):
     def _main_cxhack_cxhtml(self):
         # custom hacks for com.codeweavers.c4.cxhtml
         # occurs before install
-        if self.installer.parent.appid == 'com.codeweavers.c4.cxhtml' or \
-           self.installer.parent.appid == 'com.codeweavers.c4.7828':
+        if self.installer.parent.appid in ('com.codeweavers.c4.cxhtml', 'com.codeweavers.c4.7828'):
             engine = self.scheduler
 
             cmd = [os.path.join(cxutils.CX_ROOT, "bin", "wine"),
@@ -1123,10 +1123,8 @@ class AIECore(cxaiebase.AIETask):
         # dotnet needs to whip out the wine version before it installs
         # custom hacks for com.codeweavers.c4.597
         # occurs before install
-        if self.installer.parent.appid == 'com.codeweavers.c4.597' or \
-           self.installer.parent.appid == 'com.codeweavers.c4.1578' or \
-           self.installer.parent.appid == 'com.codeweavers.c4.5917' or \
-           self.installer.parent.appid == 'com.codeweavers.c4.17131':
+        if self.installer.parent.appid in ('com.codeweavers.c4.597', 'com.codeweavers.c4.1578',
+            'com.codeweavers.c4.5917', 'com.codeweavers.c4.17131'):
             engine = self.scheduler
             apps = appdetector.get_uninstall_applications(engine.installtask.bottlename)
             for name in ('Wine Mono', 'Wine Mono Runtime', 'Wine Mono Windows Support'):
@@ -1143,8 +1141,7 @@ class AIECore(cxaiebase.AIETask):
         # dotnet 3.0  and 3.0 sp 1 needs to remove some service keys
         # custom hacks for com.codeweavers.c4.9383 and 3356
         # occurs before install
-        if self.installer.parent.appid == 'com.codeweavers.c4.9383' or \
-           self.installer.parent.appid == 'com.codeweavers.c4.3356':
+        if self.installer.parent.appid in ('com.codeweavers.c4.9383', 'com.codeweavers.c4.3356'):
             engine = self.scheduler
             cmd = [os.path.join(cxutils.CX_ROOT, "bin", "wine"),
                    "--bottle", engine.installtask.bottlename,
@@ -1190,8 +1187,7 @@ class AIECore(cxaiebase.AIETask):
         # dotnet 2.0 sps needs to make sure a service is shut down
         # custom hacks for com.codeweavers.c4.9322 and 6945
         # occurs before install
-        if self.installer.parent.appid == 'com.codeweavers.c4.9322' or \
-           self.installer.parent.appid == 'com.codeweavers.c4.6945':
+        if self.installer.parent.appid in ('com.codeweavers.c4.9322', 'com.codeweavers.c4.6945'):
             engine = self.scheduler
 
             cmd = [os.path.join(cxutils.CX_ROOT, "bin", "wine"),
@@ -1209,8 +1205,7 @@ class AIECore(cxaiebase.AIETask):
         # dotnet 2.0 sps needs to make sure everything is shut down at the end
         # custom hacks for com.codeweavers.c4.9322 and 6945
         # occurs before install
-        if self.installer.parent.appid == 'com.codeweavers.c4.9322' or \
-           self.installer.parent.appid == 'com.codeweavers.c4.6945':
+        if self.installer.parent.appid in ('com.codeweavers.c4.9322', 'com.codeweavers.c4.6945'):
             engine = self.scheduler
 
             cmd = [os.path.join(cxutils.CX_ROOT, "bin", "wine"),
@@ -1263,8 +1258,7 @@ class AIECore(cxaiebase.AIETask):
                     'cxhack_taskkill',
                     'delete_downloaded_installer',
                    )
-        self.cleanup_lock.acquire() # If we run cleanup functions concurrently, they could return too early
-        try:
+        with self.cleanup_lock: # If we run cleanup functions concurrently, they could return too early
             for routine in routines:
                 # Don't use run_routines for this because we always want to run all of them.
                 try:
@@ -1273,8 +1267,6 @@ class AIECore(cxaiebase.AIETask):
                 except:
                     cxlog.warn("Cleanup function failed:\n%s" % traceback.format_exc())
                     result = False
-        finally:
-            self.cleanup_lock.release()
         return result
 
     def can_cancel(self):
