@@ -1,18 +1,11 @@
 # (c) Copyright 2009-2013, 2015-2018. CodeWeavers, Inc.
 
-import sys
 import os
 import os.path
 import stat
 import tempfile
-if sys.version_info >= (3,):
-    import http.cookiejar as cookielib # pylint: disable=E0401
-    import urllib.request as urllib_request # pylint: disable=E0401,E0611
-    import urllib.parse as urlparse # pylint: disable=E0401,E0611
-else:
-    import cookielib # pylint: disable=E0401
-    import urllib2 as urllib_request # pylint: disable=E0401
-    import urlparse # pylint: disable=E0401
+import hashlib
+import urllib.parse as urlparse
 
 import cxproduct
 import cxlog
@@ -286,39 +279,6 @@ def get_cached_filename(url, filename):
     md5hasher.update(cxutils.string_to_utf8(url.split('#', 1)[0]))
     return md5hasher.hexdigest() + "." + filename
 
-def validate_installer(filename):
-    """Checks whether the specified file is an html file, and if it is,
-    whether it redirects us elsewhere.
-    """
-    return cxhtmlutils.is_html(filename, 4096)
-
-def get_cached_installer(url):
-    """Checks whether we have an installer for the specified url in our
-    downloaded installers cache.
-    """
-    basename = get_cached_filename(url, "")
-    cxlog.log_("aie", "looking for " + cxlog.to_str(basename))
-    for dirpath in (cxproduct.get_user_dir(), cxproduct.get_managed_dir()):
-        try:
-            for dentry in os.listdir(os.path.join(dirpath, "installers")):
-                if dentry.startswith(basename):
-                    path = os.path.join(dirpath, "installers", dentry)
-                    if not os.path.isfile(path):
-                        continue
-                    cxlog.log_("aie", "  -> " + cxlog.to_str(path))
-                    is_html, _redirect = validate_installer(path)
-                    if not is_html:
-                        return path
-                    # A bad file got cached somehow (maybe by an old
-                    # version)
-                    cxlog.log_("aie", "  -> deleted (html file)")
-                    os.unlink(path)
-        except OSError:
-            # The directory does not exist or is not readable.
-            # Just skip to the next
-            pass
-    return None
-
 
 def _is_cache_filename(name):
     if name.startswith('local.tmp.'):
@@ -373,6 +333,37 @@ def delete_cached_installers():
         os.unlink(filename)
 
 
+def verify_checksum(filename, checksum):
+    if not checksum:
+        return True
+    for desc in checksum.split(';'):
+        if ':' not in desc:
+            continue
+        algorithm, digest = desc.split(':', 1)
+        try:
+            hashobj = hashlib.new(algorithm)
+        except ValueError:
+            # Unsupported hash algorithm, ignore it
+            cxlog.warn('Failed to create hash object for %s' % cxlog.debug_str(algorithm))
+            continue
+        with open(filename, 'rb') as f:
+            while True:
+                buf = f.read(8192)
+                if buf:
+                    hashobj.update(buf)
+                else:
+                    break
+        try:
+            actual_digest = hashobj.hexdigest()
+        except TypeError:
+            # shake_128 and shake_256 require a length to be passed in
+            actual_digest = hashobj.hexdigest(len(digest) / 2)
+        if digest.lower() != actual_digest:
+            cxlog.warn('Checksum mismatch: got %s, expected %s' % (actual_digest, digest))
+            return False
+    return True
+
+
 class AIEDownload(cxaiebase.AIETask):
 
     #####
@@ -413,19 +404,72 @@ class AIEDownload(cxaiebase.AIETask):
     #
     #####
 
-    def _geturl(self):
+    def _get_url_and_checksum(self):
         if (self.aiecore.installer.parent.app_profile.download_urls_64bit and
             self.scheduler.installtask.is64bit()):
             download_urls = self.aiecore.installer.parent.app_profile.download_urls_64bit
         else:
             download_urls = self.aiecore.installer.parent.app_profile.download_urls
-        _lang, url = cxutils.get_language_value(
+        _lang, result = cxutils.get_language_value(
             download_urls,
             self.scheduler.installtask.realInstallerLocale)
-        return url
+        return result
+
+    def _geturl(self):
+        return self._get_url_and_checksum()[0]
 
     url = property(_geturl)
 
+    def _getchecksum(self):
+        return self._get_url_and_checksum()[1]
+
+    checksum = property(_getchecksum)
+
+
+    def validate_installer(self, filename):
+        """Checks whether the specified file is valid.
+
+        Returns a tuple of:
+            True if the file is valid, otherwise False.
+            The target if the file is html with a redirect.
+            A suggested error message if the file is invalid.
+        """
+        is_html, redirect = cxhtmlutils.is_html(filename, 4096)
+        if is_html:
+            return False, redirect, _("Got an HTML file instead of an installer for %s") % self.url
+        if not verify_checksum(filename, self.checksum):
+            return False, False, _("The downloaded installer did not match the checksum for %s") % self.url
+        return True, False, None
+
+
+    def get_cached_installer(self):
+        """Checks whether we have an installer for the specified url in our
+        downloaded installers cache.
+        """
+        url = self.url
+        basename = get_cached_filename(url, "")
+        cxlog.log_("aie", "looking for " + cxlog.to_str(basename))
+        for dirpath in (cxproduct.get_user_dir(), cxproduct.get_managed_dir()):
+            try:
+                for dentry in os.listdir(os.path.join(dirpath, "installers")):
+                    if dentry.startswith(basename):
+                        path = os.path.join(dirpath, "installers", dentry)
+                        if not os.path.isfile(path):
+                            continue
+                        cxlog.log_("aie", "  -> " + cxlog.to_str(path))
+                        valid, _redirect, errormsg = self.validate_installer(path)
+                        if valid:
+                            return path
+                        # A bad file got cached somehow (maybe by an old
+                        # version)
+                        cxlog.log_("aie", "  -> deleted (%s)" % errormsg)
+                        os.unlink(path)
+            except OSError:
+                # The directory does not exist or is not readable.
+                # Just skip to the next
+                pass
+        return None
+
 
     def _needs_download(self):
         """Returns True if the installer needs to be downloaded, and False
@@ -443,7 +487,7 @@ class AIEDownload(cxaiebase.AIETask):
 
         installer_file = get_builtin_installer(self.aiecore.installer.parent.appid)
         if not installer_file and not self.aiecore.installer.always_redownload:
-            installer_file = get_cached_installer(self.url)
+            installer_file = self.get_cached_installer()
         if installer_file:
             self.aiecore.state['install_source'] = installer_file
             return False
@@ -520,6 +564,8 @@ class AIEDownload(cxaiebase.AIETask):
                 self.error = _("HTTP error 404: The file '%s' could not be found.") % cxlog.to_str(urlgetter.url)
             else:
                 self.error = _("The HTTP server returned failure code %s.") % exception.code
+        elif isinstance(exception, cxurlget.UrlError):
+            self.error = str(exception)
         else:
             self.error = cxlog.debug_str(exception)
 
@@ -533,7 +579,7 @@ class AIEDownload(cxaiebase.AIETask):
 
         # Some sites return a cookie and redirect us to another URL which
         # expects to receive that cookie. So create a temporary cookie jar.
-        cookiejar = cookielib.CookieJar()
+        cookiejar = cxurlget.UrlGetter.make_cookie_storage()
 
         installers_dir = self.get_installers_dir()
         url = self.url
@@ -542,15 +588,14 @@ class AIEDownload(cxaiebase.AIETask):
             tmpfileno, tmppath = tempfile.mkstemp(prefix='local.tmp.', dir=installers_dir)
             # fdopen() acquires the fd as its own
             tmpfile = os.fdopen(tmpfileno, "wb")
-            opener = urllib_request.build_opener(urllib_request.HTTPCookieProcessor(cookiejar))
-            self._getter = cxurlget.UrlGetter(url, tmpfile, user_agent=user_agent, notify_progress=self.urlgetter_progress, notify_failed=self.urlgetter_failed, opener=opener)
+            self._getter = cxurlget.UrlGetter(url, tmpfile, user_agent=user_agent, notify_progress=self.urlgetter_progress, notify_failed=self.urlgetter_failed, cookie_storage=cookiejar)
             self._getter.fetch() # closes tmpfile for us
             if self.canceled or self.error:
                 os.unlink(tmppath)
                 return False
 
-            is_html, redirect = validate_installer(tmppath)
-            if not is_html:
+            valid, redirect, errormsg = self.validate_installer(tmppath)
+            if valid:
                 break
 
             # We may have gotten an HTML file instead of the installer:
@@ -583,9 +628,9 @@ class AIEDownload(cxaiebase.AIETask):
             # final destination is valid
             url = urlparse.urljoin(self._getter.url, redirect)
 
-        if is_html:
+        if not valid:
             os.unlink(tmppath)
-            self.error = _("Got an HTML file instead of an installer for %s") % self.url
+            self.error = errormsg
             return False
 
         installer_file = os.path.join(installers_dir, get_cached_filename(self.url, self._getter.basename))
@@ -654,7 +699,7 @@ class AIEScanMedia(cxaiebase.AIETask):
             elif rank == best_rank[download]:
                 best_matches[download].append(match)
 
-        for download, matches in cxutils.iteritems(best_matches):
+        for download, matches in best_matches.items():
             # Reject ties
             if len(matches) > 1:
                 cxlog.log("got a tie for %s: %s" % (cxlog.to_str(download.aiecore.installer.local_installer_file_globs[best_rank[download]]), cxlog.debug_str(matches)))
