@@ -12,11 +12,7 @@ import sys
 import time
 import traceback
 
-if sys.version_info >= (3,):
-    from urllib.parse import urlparse, unquote # pylint: disable=E0611,F0401
-else:
-    from urllib import unquote # pylint: disable=E0611
-    from urlparse import urlparse # pylint: disable=F0401
+from urllib.parse import urlparse, unquote
 
 import cxlog
 import distversion
@@ -50,8 +46,7 @@ def mkdirs(path, *args, **kwargs):
     """
     try:
         os.makedirs(path, *args, **kwargs)
-    except OSError:
-        ose = sys.exc_info()[1]
+    except OSError as ose:
         if ose.errno != errno.EEXIST:
             cxlog.err("unable to create the '%s' directory: %s" % (cxlog.debug_str(path), ose.strerror))
             return False
@@ -63,7 +58,7 @@ PERMISSIONS_NOEXEC = 2
 
 def check_mmap_permissions(filename):
     try:
-        f = open(filename, 'rb')
+        f = open(filename, 'rb') # pylint: disable=R1732
     except IOError:
         return PERMISSIONS_NOREAD
     try:
@@ -78,14 +73,11 @@ def check_mmap_permissions(filename):
 
 def check_fake_dll(path):
     if os.path.exists(path):
-        sfile = open(path, "rb")
-        try:
+        with open(path, "rb") as sfile:
             sfile.seek(0x40)
             tag = sfile.read(20)
             if tag.startswith((b"Wine builtin DLL", b"Wine placeholder DLL")):
                 return True
-        finally:
-            sfile.close()
     return False
 
 #####
@@ -94,7 +86,7 @@ def check_fake_dll(path):
 #
 #####
 
-class FLock(object):
+class FLock:
 
     def __init__(self, name):
         """Creates and takes a lock called name using flock().
@@ -121,18 +113,16 @@ class FLock(object):
             st = os.lstat(self._path)
             if not stat.S_ISREG(st.st_mode):
                 raise ValueError("'%s' is not a regular file" % self._path)
-        except OSError:
-            ose = sys.exc_info()[1]
+        except OSError as ose:
             if ose.errno != errno.ENOENT:
-                raise ose
-        self._lock = open(self._path, 'w')
+                raise
+        self._lock = open(self._path, 'wb') # pylint: disable=R1732
 
         # Take the lock
         cxlog.log("%d: Grabbing the '%s' lock" % (os.getpid(), self._path))
         try:
             fcntl.flock(self._lock, fcntl.LOCK_EX)
-        except Exception:
-            e = sys.exc_info()[1]
+        except Exception as e:
             # The filesystem probably does not support locking
             cxlog.warn("unable to take the '%s' lock: %s" % (name, e))
             self._lock.close()
@@ -162,8 +152,7 @@ class FLock(object):
             try:
                 try:
                     fcntl.flock(self._lock, fcntl.LOCK_UN)
-                except Exception:
-                    e = sys.exc_info()[1]
+                except Exception as e:
                     cxlog.warn("unable to release the '%s' lock: %s" % (self._name, e))
             finally:
                 self._lock.close()
@@ -176,137 +165,40 @@ class FLock(object):
 #
 #####
 
-try:
-    # pylint: disable=E0602
-    bytes_type = bytes
-except NameError:
-    bytes_type = str
+def string_to_unicode(string):
+    if isinstance(string, str):
+        return string
+    if isinstance(string, bytes):
+        return string.decode('utf8', 'surrogateescape')
+    try:
+        m = memoryview(string)
+        return m.tobytes().decode('utf-8', 'surrogateescape')
+    except (NameError, TypeError):
+        raise ValueError("string_to_unicode called with non-string") #pylint: disable=W0707
 
-try:
-    unicode_type = unicode
-except NameError:
-    unicode_type = str
+def string_to_utf8(string):
+    if isinstance(string, str):
+        return string.encode('utf8', 'surrogateescape')
+    if isinstance(string, bytes):
+        return string
+    try:
+        m = memoryview(string)
+        return m.tobytes()
+    except (NameError, TypeError):
+        raise ValueError("string_to_utf8 called with non-string") #pylint: disable=W0707
 
-if sys.version_info < (3,):
-    def string_to_unicode(string):
-        if isinstance(string, unicode_type):
-            return string
-        if isinstance(string, bytes_type):
-            return string.decode('utf8')
-        try:
-            m = memoryview(string)
-            return m.tobytes().decode('utf-8')
-        except (NameError, TypeError):
-            raise ValueError("string_to_unicode called with non-string")
 
-    def string_to_utf8(string):
-        if isinstance(string, unicode_type):
-            return string.encode('utf8')
-        if isinstance(string, bytes_type):
-            return string
-        try:
-            m = memoryview(string)
-            return m.tobytes()
-        except (NameError, TypeError):
-            raise ValueError("string_to_utf8 called with non-string")
+string_to_str = string_to_unicode
 
-    def b(x):
-        return x
 
-    def iterkeys(d):
-        return d.iterkeys()
-
-    def iteritems(d):
-        return d.iteritems()
-
-    def itervalues(d):
-        return d.itervalues()
-
-    def keys(d):
-        return d.keys()
-
-    def items(d):
-        return d.items()
-
-    def values(d):
-        return d.values()
-
-    def open_text(path):
-        "opens a file for reading with newline conversion"
-        return open(path, 'U')
-
-    def is_file_not_found(exception):
-        return isinstance(exception, IOError) and exception.errno == 2
-
-    cmp = cmp # pylint: disable=W0622,E0602,W0127
-    range = xrange # pylint: disable=W0622,E0602
-    long = long # pylint: disable=W0622,E0602,W0127
-    string_to_str = string_to_utf8
-else:
-    def string_to_unicode(string):
-        if isinstance(string, unicode_type):
-            return string
-        if isinstance(string, bytes_type):
-            return string.decode('utf8', 'surrogateescape')
-        try:
-            m = memoryview(string)
-            return m.tobytes().decode('utf-8', 'surrogateescape')
-        except (NameError, TypeError):
-            raise ValueError("string_to_unicode called with non-string")
-
-    def string_to_utf8(string):
-        if isinstance(string, unicode_type):
-            return string.encode('utf8', 'surrogateescape')
-        if isinstance(string, bytes_type):
-            return string
-        try:
-            m = memoryview(string)
-            return m.tobytes()
-        except (NameError, TypeError):
-            raise ValueError("string_to_utf8 called with non-string")
-
-    def b(x):
-        if isinstance(x, int):
-            return bytes([x])
-        return x.encode('utf8')
-
-    def iterkeys(d):
-        return iter(d.keys())
-
-    def iteritems(d):
-        return iter(d.items())
-
-    def itervalues(d):
-        return iter(d.values())
-
-    def keys(d):
-        return list(d.keys())
-
-    def items(d):
-        return list(d.items())
-
-    def values(d):
-        return list(d.values())
-
-    def cmp(v1, v2):
-        return (v1 > v2) - (v1 < v2)
-
-    def open_text(path):
-        "opens a file for reading with newline conversion"
-        return open(path, 'rt')
-
-    def is_file_not_found(exception):
-        return isinstance(exception, FileNotFoundError) # pylint: disable=E0602
-
-    range = range # pylint: disable=W0622,E0602,W0127,R0204
-    long = int
-    string_to_str = string_to_unicode
+def cmp(v1, v2):
+    return (v1 > v2) - (v1 < v2)
 
 
 def expect_unicode(string):
-    if isinstance(string, unicode_type):
+    if isinstance(string, str):
         return string
-    if isinstance(string, bytes_type):
+    if isinstance(string, bytes):
         cxlog.warn("Byte string passed where unicode string was expected:\n%s" % ''.join(traceback.format_stack()[0:-1]))
         return string.decode('utf8')
     raise ValueError("expect_unicode() called with non-string")
@@ -337,12 +229,6 @@ def shquote_string(string):
     string = string.replace("\'", "\\\'")
     return '"' + string + '"'
 
-_BAD_SHELL_CHARS = re.compile(r"[^a-zA-Z0-9\/.,+_-]")
-def maybe_shquote_string(string):
-    if _BAD_SHELL_CHARS.search(string):
-        return shquote_string(string)
-    return string
-
 def unmangle(string):
     result = []
     x = 0
@@ -480,28 +366,16 @@ def sanitize_bottlename(name_hint):
         cxlog.err("sanitize_bottlename() generated an invalid bottle name: '%s'" % bottlename)
     return bottlename
 
-if sys.version_info >= (3,):
-    def path_to_uri(path):
-        result = ['file://']
-        # FIXME: use filesystem encoding?
-        path = string_to_utf8(path)
-        for char in path:
-            if char >= 0x80 or char in b'%#?*!&=()[],:;@$+ ':
-                result.append('%%%02X' % char)
-            else:
-                result.append(chr(char))
-        return ''.join(result)
-else:
-    def path_to_uri(path):
-        result = ['file://']
-        # FIXME: use filesystem encoding?
-        path = string_to_utf8(path)
-        for char in path:
-            if ord(char) >= 0x80 or char in '%#?*!&=()[],:;@$+ ':
-                result.append('%%%02X' % ord(char))
-            else:
-                result.append(char)
-        return ''.join(result)
+def path_to_uri(path):
+    result = ['file://']
+    # FIXME: use filesystem encoding?
+    path = string_to_utf8(path)
+    for char in path:
+        if char >= 0x80 or char in b'%#?*!&=()[],:;@$+ ':
+            result.append('%%%02X' % char)
+        else:
+            result.append(chr(char))
+    return ''.join(result)
 
 def uri_to_path(uri):
     scheme, _netloc, path, _params, _query, _fragment = urlparse(uri)
@@ -633,38 +507,6 @@ def dirname(path):
     return ''
 
 
-_regex_specials = set('.^$*+?[]|()')
-
-def unescape_regex(string):
-    """If a regex can match multiple strings, return None. If it can match only
-    one string, this function MAY return that string."""
-    in_curly_brace = False
-    escape = False
-    result = []
-    string = string.lstrip('^').rstrip('$')
-    for ch in string:
-        if ch == '{' and not escape:
-            in_curly_brace = True
-        elif in_curly_brace:
-            if ch == '}':
-                return None
-            if not ch.isdigit() and ch != ',':
-                in_curly_brace = False
-
-        if escape:
-            if ch.isalnum():
-                # An escaped letter or number might have special meaning
-                return None
-            result.append(ch)
-            escape = False
-        elif ch in _regex_specials:
-            return None
-        elif ch == '\\':
-            escape = True
-        else:
-            result.append(ch)
-    return ''.join(result)
-
 _OCTALCHAR_RE = None
 def expand_octal_chars(string):
     r"""Expand escaped octal characters of the form '\040' in the specified
@@ -677,7 +519,7 @@ def expand_octal_chars(string):
     if not _OCTALCHAR_RE:
         _OCTALCHAR_RE = re.compile('(\\\\[0-7]{3})')
     i = 0
-    expanded = list()
+    expanded = []
     for match in _OCTALCHAR_RE.finditer(string):
         expanded.append(string[i:match.start()])
         expanded.append(chr(int(string[match.start()+1:match.end()], 8)))
@@ -700,7 +542,7 @@ def expand_unix_string(environ, string):
     if not _UNIXVAR_RE:
         _UNIXVAR_RE = re.compile(r'\${(\w+)}')
     i = 0
-    expanded = list()
+    expanded = []
     for match in _UNIXVAR_RE.finditer(string):
         expanded.append(string[i:match.start()])
         if match.group(1) in environ:
@@ -793,6 +635,10 @@ def version_as_tuple():
 
 _SINGLETON_RE = re.compile("(-x)?-[a-z]$")
 _PREFERRED_LANGUAGES = {}
+_LANGUAGE_ASSOCIATIONS = {
+    'zh-hans':'zh-cn',
+    'zh-hant':'zh-tw',
+}
 def get_preferred_languages(language=None):
     """Returns a sequence of the user's preferred languages if language is
     None; and a sequence of variants otherwise.
@@ -851,6 +697,11 @@ def get_preferred_languages(language=None):
         preferred.insert(fallback[1], fallback[0])
         seen.add(fallback[0])
 
+        if fallback[0] in _LANGUAGE_ASSOCIATIONS:
+            associated_lang = _LANGUAGE_ASSOCIATIONS[fallback[0]]
+            preferred.insert(fallback[1] + 1, associated_lang)
+            seen.add(associated_lang)
+
     # Add the ultimate fallback
     preferred.append('')
     _PREFERRED_LANGUAGES[language] = preferred
@@ -879,8 +730,7 @@ def get_language_value(dictionary, language=None):
     if dictionary:
         # The dictionary is not empty, but there is no default. Just pick
         # something, predictably.
-        languages = keys(dictionary)
-        languages.sort()
+        languages = sorted(dictionary)
         for lang in languages:
             if dictionary[lang]:
                 return (lang, dictionary[lang])
@@ -902,8 +752,6 @@ def setup_textdomain():
             locale.bindtextdomain('crossover', _LOCALE_PATH)
         import gettext
         gettext.bindtextdomain("crossover", _LOCALE_PATH)
-        if sys.version_info < (3,):
-            gettext.bind_textdomain_codeset("crossover", "UTF-8") # pylint: disable=W1505
         gettext.textdomain("crossover")
 
 
@@ -972,7 +820,7 @@ def run(args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None,
     if stdin is None:
         subin = None
     elif stdin == NULL:
-        subin = open("/dev/null", "r")
+        subin = open("/dev/null", "rb") # pylint: disable=R1732
     else:
         raise TypeError("invalid value for stdin")
     if stdout:
@@ -981,7 +829,7 @@ def run(args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None,
             if stdout == GRAB:
                 out_log = True
         elif stdout == NULL:
-            subout = open("/dev/null", "w")
+            subout = open("/dev/null", "wb") # pylint: disable=R1732
         else:
             raise TypeError("invalid value for stdout")
     else:
@@ -999,7 +847,7 @@ def run(args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None,
             if cxlog.is_on():
                 suberr = None
             else:
-                suberr = open("/dev/null", "w")
+                suberr = open("/dev/null", "wb") # pylint: disable=R1732
         else:
             raise TypeError("invalid value for stderr")
     else:
@@ -1011,7 +859,7 @@ def run(args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None,
             if childpid == 0:
                 #child
                 try:
-                    subp = subprocess.Popen(args, bufsize, executable, subin, subout,
+                    subp = subprocess.Popen(args, bufsize, executable, subin, subout, # pylint: disable=R1732
                                             suberr, preexec_fn, close_fds=True,
                                             shell=shell, cwd=cwd, env=env,
                                             universal_newlines=universal_newlines)
@@ -1025,14 +873,13 @@ def run(args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None,
                 out = None
                 err = None
         else:
-            subp = subprocess.Popen(args, bufsize, executable, subin, subout,
+            subp = subprocess.Popen(args, bufsize, executable, subin, subout, # pylint: disable=R1732
                                     suberr, preexec_fn, close_fds=True,
                                     shell=shell, cwd=cwd, env=env,
                                     universal_newlines=universal_newlines)
             out, err = subp.communicate()
             retcode = subp.wait()
-    except OSError:
-        exception = sys.exc_info()[1]
+    except OSError as exception:
         retcode = -1
         out = ""
         err = cxlog.to_str(exception)
@@ -1104,8 +951,7 @@ def select(iwtd, owtd, ewtd, timeout=None):
     while True:
         try:
             return select_select(iwtd, owtd, ewtd, timeout)
-        except select_error:
-            err = sys.exc_info()[1]
+        except select_error as err:
             if err.args[0] == errno.EINTR:
                 if timeout is not None:
                     timeout = 0
@@ -1139,7 +985,7 @@ def _internal_load_xdg_dirs():
         config_file = os.path.join(os.environ.get('HOME', '/'), ".config", "user-dirs.dirs")
     result = {}
     try:
-        f = open_text(config_file)
+        f = open(config_file, 'rt', encoding='utf8', errors='surrogateescape') # pylint: disable=R1732
     except:
         return {}
     try:
