@@ -44,7 +44,7 @@ class error(Exception):
 class MissingDiskError(error):
     pass
 
-class Volume(object):
+class Volume:
     device = ''
     # A string that uniquely identifies this device and does not change during
     # its lifetime. If possible, this is the filename of a block device.
@@ -104,7 +104,7 @@ class UDisks2Volume(Volume):
 def char_array_to_str(a):
     return ''.join(chr(x) for x in a).split('\0', 1)[0]
 
-class UDisks2MountPoints(object):
+class UDisks2MountPoints:
 
     def __init__(self, add_notify=not_much, del_notify=not_much):
         self.devices = {}
@@ -129,7 +129,7 @@ class UDisks2MountPoints(object):
         self.signals.append(self.udisks.connect_to_signal("InterfacesRemoved", self.interfaces_removed))
         self.signals.append(system_bus.add_signal_receiver(self.properties_changed, 'PropertiesChanged', 'org.freedesktop.DBus.Properties', 'org.freedesktop.UDisks2', path_keyword='objpath'))
 
-        for objpath, info in cxutils.iteritems(self.udisks.GetManagedObjects()):
+        for objpath, info in self.udisks.GetManagedObjects().items():
             self.interfaces_added(objpath, info)
 
     def get_volume(self, objpath):
@@ -186,7 +186,7 @@ class UDisks2MountPoints(object):
             known_props = {}
             volume.interface_properties[interface_name] = known_props
 
-        for name, value in cxutils.iteritems(props):
+        for name, value in props.items():
             known_props[name] = value
             self.property_changed(objpath, interface_name, name)
 
@@ -213,7 +213,7 @@ class UDisks2MountPoints(object):
         self.report_changes()
 
     def interfaces_added(self, objpath, info):
-        for interface_name, props in cxutils.iteritems(info):
+        for interface_name, props in info.items():
             self._properties_changed(interface_name, props, (), objpath)
 
         self.report_changes()
@@ -243,7 +243,7 @@ class UDisks2MountPoints(object):
         return volume.mounted
 
     def report_changes(self):
-        for volume in cxutils.itervalues(self.devices):
+        for volume in self.devices.values():
             if self.should_report_volume(volume) != volume.is_published:
                 if not volume.is_published:
                     self.add_notify(volume)
@@ -268,7 +268,7 @@ class UDisks2MountPoints(object):
 class UDisksVolume(Volume):
     objpath = ''
 
-class UDisksMountPoints(object):
+class UDisksMountPoints:
     def __init__(self, add_notify=not_much, del_notify=not_much):
         self.devices = {}
 
@@ -325,7 +325,7 @@ class HALVolume(Volume):
     udi = ''
     # The HAL unique device id
 
-class HALMountPoints(object):
+class HALMountPoints:
     def new_device(self, udi, *_rest):
         #  Get the device object
         device_obj = system_bus.get_object('org.freedesktop.Hal', udi)
@@ -405,7 +405,7 @@ class HALMountPoints(object):
         self.del_notify = not_much
         for signal in self.signals:
             signal.remove()
-        for signal in cxutils.values(self.device_signals):
+        for signal in self.device_signals.values():
             signal.remove()
 
     def __del__(self):
@@ -415,7 +415,7 @@ class UnixVolume(Volume):
     # A mount point obtained from the mount command or /etc/fstab.
     pass
 
-class FailsafeMountPoints(object):
+class FailsafeMountPoints:
 
     # The Linux mtab format assumes device paths don't contain spaces.
     # Otherwise it is ambiguous.
@@ -428,9 +428,8 @@ class FailsafeMountPoints(object):
         mountdata = None
         for filename in ('/etc/mtab', '/etc/mnttab'):
             try:
-                thisfile = open(filename, 'r')
-                mountdata = list(thisfile.readlines())
-                thisfile.close()
+                with open(filename, 'r', encoding='utf8', errors='surrogateescape') as thisfile: # pylint: disable=W1514
+                    mountdata = list(thisfile.readlines())
                 regexps = (self.LINUX_MTAB_RE, self.SOLARIS_MTAB_RE)
                 break
             except IOError as ioerror:
@@ -444,7 +443,7 @@ class FailsafeMountPoints(object):
             mountdata = out.split('\n')
             regexps = (self.LINUX_MOUNT_RE, self.FREEBSD_MOUNT_RE)
 
-        known_devices = cxutils.keys(self.devices)
+        known_devices = list(self.devices.keys())
 
         # Given the call parameters, communicate() always returns a string
         for line in mountdata:
@@ -518,7 +517,7 @@ notifier_classes = [
     FailsafeMountPoints,
     ]
 
-class MountPointsNotifier(object):
+class MountPointsNotifier:
     "A class that returns the combined results from the full set of notifiers"
     def __init__(self, add_notify=not_much, del_notify=not_much):
         self.add_notify = add_notify
@@ -586,6 +585,6 @@ class MountPointsNotifier(object):
 
 def GetVolumeList():
     mountpointsnotify = MountPointsNotifier()
-    result = cxutils.values(mountpointsnotify.devices)
+    result = list(mountpointsnotify.devices.values())
     mountpointsnotify.close()
     return result
